@@ -7,7 +7,7 @@
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/commands/async.c,v 1.102 2003/10/16 16:50:41 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/commands/async.c,v 1.102.2.2 2005/08/25 22:07:15 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -52,9 +52,10 @@
  *	  transaction, since by assumption it is only called from outside any
  *	  transaction.
  *
- * Although we grab AccessExclusiveLock on pg_listener for any operation,
+ * Although we grab ExclusiveLock on pg_listener for any operation,
  * the lock is never held very long, so it shouldn't cause too much of
- * a performance problem.
+ * a performance problem.  (Previously we used AccessExclusiveLock, but
+ * there's no real reason to forbid concurrent reads.)
  *
  * An application that listens on the same relname it notifies will get
  * NOTIFY messages for its own NOTIFYs.  These can be ignored, if not useful,
@@ -196,7 +197,7 @@ Async_Listen(char *relname, int pid)
 	if (Trace_notify)
 		elog(DEBUG1, "Async_Listen(%s,%d)", relname, pid);
 
-	lRel = heap_openr(ListenerRelationName, AccessExclusiveLock);
+	lRel = heap_openr(ListenerRelationName, ExclusiveLock);
 
 	/* Detect whether we are already listening on this relname */
 	scan = heap_beginscan(lRel, SnapshotNow, 0, (ScanKey) NULL);
@@ -216,7 +217,7 @@ Async_Listen(char *relname, int pid)
 
 	if (alreadyListener)
 	{
-		heap_close(lRel, AccessExclusiveLock);
+		heap_close(lRel, ExclusiveLock);
 		return;
 	}
 
@@ -244,7 +245,7 @@ Async_Listen(char *relname, int pid)
 
 	heap_freetuple(tuple);
 
-	heap_close(lRel, AccessExclusiveLock);
+	heap_close(lRel, ExclusiveLock);
 
 	/*
 	 * now that we are listening, make sure we will unlisten before dying.
@@ -290,7 +291,7 @@ Async_Unlisten(char *relname, int pid)
 	if (Trace_notify)
 		elog(DEBUG1, "Async_Unlisten(%s,%d)", relname, pid);
 
-	lRel = heap_openr(ListenerRelationName, AccessExclusiveLock);
+	lRel = heap_openr(ListenerRelationName, ExclusiveLock);
 
 	scan = heap_beginscan(lRel, SnapshotNow, 0, (ScanKey) NULL);
 	while ((tuple = heap_getnext(scan, ForwardScanDirection)) != NULL)
@@ -312,7 +313,7 @@ Async_Unlisten(char *relname, int pid)
 	}
 	heap_endscan(scan);
 
-	heap_close(lRel, AccessExclusiveLock);
+	heap_close(lRel, ExclusiveLock);
 
 	/*
 	 * We do not complain about unlistening something not being listened;
@@ -348,7 +349,7 @@ Async_UnlistenAll(void)
 	if (Trace_notify)
 		elog(DEBUG1, "Async_UnlistenAll");
 
-	lRel = heap_openr(ListenerRelationName, AccessExclusiveLock);
+	lRel = heap_openr(ListenerRelationName, ExclusiveLock);
 	tdesc = RelationGetDescr(lRel);
 
 	/* Find and delete all entries with my listenerPID */
@@ -362,7 +363,7 @@ Async_UnlistenAll(void)
 		simple_heap_delete(lRel, &lTuple->t_self);
 
 	heap_endscan(scan);
-	heap_close(lRel, AccessExclusiveLock);
+	heap_close(lRel, ExclusiveLock);
 }
 
 /*
@@ -457,7 +458,7 @@ AtCommit_Notify(void)
 	value[0] = value[1] = value[2] = (Datum) 0;
 	value[Anum_pg_listener_notify - 1] = Int32GetDatum(MyProcPid);
 
-	lRel = heap_openr(ListenerRelationName, AccessExclusiveLock);
+	lRel = heap_openr(ListenerRelationName, ExclusiveLock);
 	tdesc = RelationGetDescr(lRel);
 	scan = heap_beginscan(lRel, SnapshotNow, 0, (ScanKey) NULL);
 
@@ -511,8 +512,9 @@ AtCommit_Notify(void)
 			}
 			else if (listener->notification == 0)
 			{
-				ItemPointerData ctid;
 				int			result;
+				ItemPointerData update_ctid;
+				TransactionId update_xmax;
 
 				rTuple = heap_modifytuple(lTuple, lRel,
 										  value, nulls, repl);
@@ -532,7 +534,7 @@ AtCommit_Notify(void)
 				 * heap_getnext and heap_update calls.
 				 */
 				result = heap_update(lRel, &lTuple->t_self, rTuple,
-									 &ctid,
+									 &update_ctid, &update_xmax,
 									 GetCurrentCommandId(), SnapshotAny,
 									 false /* no wait for commit */);
 				switch (result)
@@ -812,7 +814,7 @@ ProcessIncomingNotify(void)
 
 	StartTransactionCommand();
 
-	lRel = heap_openr(ListenerRelationName, AccessExclusiveLock);
+	lRel = heap_openr(ListenerRelationName, ExclusiveLock);
 	tdesc = RelationGetDescr(lRel);
 
 	/* Scan only entries with my listenerPID */
